#pragma once
#if defined VERTEX_SHADER
    #ifndef sc_TextureRenderingLayout_Regular
        #define sc_TextureRenderingLayout_Regular 0
        #define sc_TextureRenderingLayout_StereoInstancedClipped 1
        #define sc_TextureRenderingLayout_StereoMultiview 2
    #endif

#define depthToGlobal depthScreenToViewSpace

#define depthToLocal depthViewToScreenSpace

#include <required2.glsl>
#include <std2_stereo_view_index.glsl>
#include <std2_texture_sub.glsl>


//SG_REFLECTION_BEGIN
//SG_REFLECTION_END



vec4 sc_textureLodLegacy(highp sampler2D tex,vec2 texSize,vec2 uv,float lod)
{
    #if sc_CanUseTextureLod==1
        return texture2DLod(tex,uv,lod);
    #elif sc_CanUseStandardDerivatives==1
        vec2 texels=uv*texSize;
        float dudx=dFdx(texels.x);
        float dvdx=dFdx(texels.y);
        float dudy=dFdy(texels.x);
        float dvdy=dFdy(texels.y);
        float rho=max(length(vec2(dudx,dvdx)),length(vec2(dudy,dvdy)));
        float mu=max(abs(dudx),abs(dudy));
        float mv=max(abs(dvdx),abs(dvdy));
        float rho2=max(mu,mv);
        float mip=log2(rho2);
        float bias=lod-mip;
        return texture2D(tex,uv,bias);
    #elif defined(FRAGMENT_SHADER)
        return texture2D(tex,uv,lod);
    #else
        return vec4(0.0);
    #endif

}

vec4 sc_textureLodLegacyArr(highp sampler2DArray tex,vec2 texSize,vec3 uv,float lod)
{
    #if sc_CanUseSampler2DArray==1
        return texture2DLod(tex,uv,lod);
    #else
        return vec4(0.0);
    #endif

}

vec4 sc_textureLod(highp sampler2D tex,vec2 texSize,vec2 uv,float lod)
{
vec2 param=texSize;
vec2 param_1=uv;
float param_2=lod;
return sc_textureLodLegacy(tex,param,param_1,param_2);
}

vec4 sc_textureLod(highp sampler2DArray tex,vec2 texSize,vec3 uv,float lod)
{
vec2 param=texSize;
vec3 param_1=uv;
float param_2=lod;
return sc_textureLodLegacyArr(tex,param,param_1,param_2);
}

vec4 sc_textureBiasOrLod(highp sampler2D tex,vec2 texSize,vec2 uv,float biasOrLod)
{
vec2 param=texSize;
vec2 param_1=uv;
float param_2=biasOrLod;
return sc_textureLod(tex,param,param_1,param_2);
}

vec4 sc_textureBiasOrLod(highp sampler2DArray tex,vec2 texSize,vec3 uv,float biasOrLod)
{
vec2 param=texSize;
vec3 param_1=uv;
float param_2=biasOrLod;
return sc_textureLod(tex,param,param_1,param_2);
}

vec4 sc_textureLod(highp sampler2D tex,vec2 uv,float lod)
{
#if (__VERSION__>=300)
    return textureLod(tex,uv,lod);
#elif defined(FRAGMENT_SHADER)
    #if sc_CanUseTextureLod
        return texture2DLod(tex,uv,lod);
    #else
        return texture2D(tex,uv,lod);
    #endif
#elif defined(VERTEX_SHADER)
    #if sc_MaxVertexTextureImageUnits>0
        return texture2DLod(tex,uv,lod);
    #else
        return vec4(0.0);
    #endif
#endif

}

vec4 sc_textureLod(highp sampler2DArray tex,vec3 uv,float lod)
{
#if (__VERSION__>=300)
    return textureLod(tex,uv,lod);
#elif (__VERSION__==120)
    return texture2DArrayLod(tex,uv,lod);
#else
    return vec4(0.0);
#endif

}

vec4 sc_textureLod(highp sampler3D tex,vec3 uv,float lod)
{
#if (__VERSION__>=300)
    return textureLod(tex,uv,lod);
#elif (__VERSION__==120)
    return texture3DLod(tex,uv,lod);
#else
    return vec4(0.0);
#endif

}

vec4 sc_textureLod(highp samplerCube tex,vec3 uv,float lod)
{
#if __VERSION__>=300
    return textureLod(tex,uv,lod);
#elif __VERSION__==100
    return vec4(0.0);
#elif defined(FRAGMENT_SHADER)
    #if sc_CanUseTextureLod
        return textureCubeLod(tex,uv,lod);
    #else
        return textureCube(tex,uv,lod);
    #endif
#elif defined(VERTEX_SHADER)
    #if sc_MaxVertexTextureImageUnits>0
        return textureCubeLod(tex,uv,lod);
    #else
        return vec4(0.0);
    #endif
#endif

}

vec2 sc_SamplingCoordsGlobalToView(inout vec3 uvi,int renderingLayout,int viewIndex)
{
if (renderingLayout==1)
{
uvi.y=((2.0*uvi.y)+float(viewIndex))-1.0;
}
return uvi.xy;
}

vec2 sc_ScreenCoordsGlobalToView(vec2 uv)
{
#if (sc_StereoRenderingMode==1)
{
vec3 uvi=vec3(uv,0.0);
int instancedClippedLayout=1;
int viewIndex=sc_GetStereoViewIndex();
vec3 param=vec3(uv,0.0);
int param_1=instancedClippedLayout;
int param_2=viewIndex;
vec2 _337=sc_SamplingCoordsGlobalToView(param,param_1,param_2);
return _337;
}
#else
{
return uv;
}
#endif
}

vec3 sc_SamplingCoordsViewToGlobal(vec2 uv,int renderingLayout,int viewIndex)
{
if (renderingLayout==0)
{
return vec3(uv,0.0);
}
else
{
if (renderingLayout==1)
{
return vec3(uv.x,(uv.y*0.5)+(0.5-(float(viewIndex)*0.5)),0.0);
}
else
{
return vec3(uv,float(viewIndex));
}
}
}

vec4 sc_SampleViewLevel(highp sampler2D tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex,float level)
{
vec2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
vec3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
vec2 param_3=texSize;
vec2 param_4=uvi.xy;
float param_5=level;
return sc_textureLod(tex,param_3,param_4,param_5);
}

vec4 sc_SampleView(highp sampler2D tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex,float bias)
{
vec2 param=texSize;
vec2 param_1=uv;
int param_2=renderingLayout;
int param_3=viewIndex;
float param_4=bias;
return sc_SampleViewLevel(tex,param,param_1,param_2,param_3,param_4);
}

vec4 sc_SampleView(highp sampler2D tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex)
{
vec2 param=texSize;
vec2 param_1=uv;
int param_2=renderingLayout;
int param_3=viewIndex;
float param_4=0.0;
return sc_SampleView(tex,param,param_1,param_2,param_3,param_4);
}

vec4 sc_SampleViewLevel(highp sampler2DArray tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex,float level)
{
vec2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
vec3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
vec2 param_3=texSize;
vec3 param_4=uvi;
float param_5=level;
return sc_textureLod(tex,param_3,param_4,param_5);
}

vec4 sc_SampleView(highp sampler2DArray tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex,float bias)
{
vec2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
vec3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
vec2 param_3=texSize;
vec3 param_4=uvi;
float param_5=bias;
return sc_textureBiasOrLod(tex,param_3,param_4,param_5);
}

vec4 sc_SampleView(highp sampler2DArray tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex)
{
vec2 param=texSize;
vec2 param_1=uv;
int param_2=renderingLayout;
int param_3=viewIndex;
float param_4=0.0;
return sc_SampleView(tex,param,param_1,param_2,param_3,param_4);
}

void sc_ClampUV(inout float value,float minValue,float maxValue,bool useClampToBorder,inout float clampToBorderFactor)
{
float clampedValue=clamp(value,minValue,maxValue);
float factor=step(abs(value-clampedValue),1e-05);
clampToBorderFactor*=(factor+((1.0-float(useClampToBorder))*(1.0-factor)));
value=clampedValue;
}

vec2 sc_platformUVFlip(vec2 uv)
{
return uv;
}

void sc_SoftwareWrap(inout float uv,int softwareWrapMode,bool useClampToBorder,inout float clampToBorderFactor)
{
if ((softwareWrapMode==0)||(softwareWrapMode==3))
{
float param=uv;
float param_1=0.0;
float param_2=1.0;
bool param_3=useClampToBorder;
float param_4=clampToBorderFactor;
sc_ClampUV(param,param_1,param_2,param_3,param_4);
uv=param;
clampToBorderFactor=param_4;
}
else
{
if (softwareWrapMode==1)
{
uv=fract(uv);
}
else
{
if (softwareWrapMode==2)
{
float uvFract=fract(uv);
float uvInt=uv-uvFract;
float uvOdd=step(0.25,fract(uvInt*0.5));
uv=mix(uvFract,1.0-uvFract,clamp(uvOdd,0.0,1.0));
}
}
}
}

vec2 quantizeUV(vec2 uv)
{
return uv;
}

vec4 sc_SampleTexture(highp sampler2D sampler_,vec2 samplerDims,int renderingLayout,int viewIndex,vec2 uv,bool useUvTransform,mat3 uvTransform,ivec2 softwareWrapModes,bool useUvMinMax,vec4 uvMinMax,bool useClampToBorder,vec4 borderColor,float mipBias)
{
vec2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=vec2((uvTransform*vec3(_uv,1.0)).xy);
}
vec2 param_18=_uv;
_uv=quantizeUV(param_18);
vec2 param_19=_uv;
_uv=sc_platformUVFlip(param_19);
vec2 param_20=samplerDims;
vec2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=mipBias;
vec4 tex=sc_SampleView(sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,vec4(clampToBorderFactor));
}
return tex;
}

vec4 sc_SampleTexture(highp sampler2DArray sampler_,vec2 samplerDims,int renderingLayout,int viewIndex,vec2 uv,bool useUvTransform,mat3 uvTransform,ivec2 softwareWrapModes,bool useUvMinMax,vec4 uvMinMax,bool useClampToBorder,vec4 borderColor,float mipBias)
{
vec2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=vec2((uvTransform*vec3(_uv,1.0)).xy);
}
vec2 param_18=_uv;
_uv=quantizeUV(param_18);
vec2 param_19=_uv;
_uv=sc_platformUVFlip(param_19);
vec2 param_20=samplerDims;
vec2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=mipBias;
vec4 tex=sc_SampleView(sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,vec4(clampToBorderFactor));
}
return tex;
}

vec4 sampleTextureWithTransform(highp sampler2DArray samp,inout vec3 uv,bool useLOD,float lod,bool useUvTransform,mat3 uvTransform)
{
if (useUvTransform)
{
vec2 _791=vec2((uvTransform*vec3(uv.xy,1.0)).xy);
uv=vec3(_791.x,_791.y,uv.z);
}
vec2 param=uv.xy;
vec2 _797=quantizeUV(param);
uv=vec3(_797.x,_797.y,uv.z);
if (useLOD)
{
vec3 param_1=uv;
float param_2=lod;
return sc_textureLod(samp,param_1,param_2);
}
else
{
return texture2D(samp,uv);
}
}

float depthScreenToViewSpace(inout float depth,vec2 projectionMatrixTerms)
{
float m22=projectionMatrixTerms.x;
float m32=projectionMatrixTerms.y;
depth=(depth*2.0)-1.0;
return m32/((-depth)-m22);
}

float depthViewToScreenSpace(inout float depth,vec2 projectionMatrixTerms)
{
float m22=projectionMatrixTerms.x;
float m32=projectionMatrixTerms.y;
float _840;
if (depth!=0.0)
{
_840=(-m22)-(m32/depth);
}
else
{
_840=0.0;
}
depth=_840;
return (depth*0.5)+0.5;
}

vec4 sampleTextureWithTransform(highp sampler2D samp,inout vec2 uv,bool useUvTransform,mat3 uvTransform)
{
if (useUvTransform)
{
uv=vec2((uvTransform*vec3(uv,1.0)).xy);
}
vec2 param=uv;
uv=quantizeUV(param);
return texture2D(samp,uv);
}

#elif defined FRAGMENT_SHADER // #if defined VERTEX_SHADER
    #ifndef sc_TextureRenderingLayout_Regular
        #define sc_TextureRenderingLayout_Regular 0
        #define sc_TextureRenderingLayout_StereoInstancedClipped 1
        #define sc_TextureRenderingLayout_StereoMultiview 2
    #endif

#define depthToGlobal depthScreenToViewSpace

#define depthToLocal depthViewToScreenSpace

#include <required2.glsl>
#include <std2_stereo_view_index.glsl>
#include <std2_texture_sub.glsl>


//SG_REFLECTION_BEGIN
//SG_REFLECTION_END



vec4 sc_textureLodLegacy(highp sampler2D tex,vec2 texSize,vec2 uv,float lod)
{
    #if sc_CanUseTextureLod==1
        return texture2DLod(tex,uv,lod);
    #elif sc_CanUseStandardDerivatives==1
        vec2 texels=uv*texSize;
        float dudx=dFdx(texels.x);
        float dvdx=dFdx(texels.y);
        float dudy=dFdy(texels.x);
        float dvdy=dFdy(texels.y);
        float rho=max(length(vec2(dudx,dvdx)),length(vec2(dudy,dvdy)));
        float mu=max(abs(dudx),abs(dudy));
        float mv=max(abs(dvdx),abs(dvdy));
        float rho2=max(mu,mv);
        float mip=log2(rho2);
        float bias=lod-mip;
        return texture2D(tex,uv,bias);
    #elif defined(FRAGMENT_SHADER)
        return texture2D(tex,uv,lod);
    #else
        return vec4(0.0);
    #endif

}

vec4 sc_textureLodLegacyArr(highp sampler2DArray tex,vec2 texSize,vec3 uv,float lod)
{
    #if sc_CanUseSampler2DArray==1
        return texture2DLod(tex,uv,lod);
    #else
        return vec4(0.0);
    #endif

}

vec4 sc_textureLod(highp sampler2D tex,vec2 texSize,vec2 uv,float lod)
{
vec2 param=texSize;
vec2 param_1=uv;
float param_2=lod;
return sc_textureLodLegacy(tex,param,param_1,param_2);
}

vec4 sc_textureLod(highp sampler2DArray tex,vec2 texSize,vec3 uv,float lod)
{
vec2 param=texSize;
vec3 param_1=uv;
float param_2=lod;
return sc_textureLodLegacyArr(tex,param,param_1,param_2);
}

vec4 sc_textureBiasOrLod(highp sampler2D tex,vec2 texSize,vec2 uv,float biasOrLod)
{
return texture2D(tex,uv,biasOrLod);
}

vec4 sc_textureBiasOrLod(highp sampler2DArray tex,vec2 texSize,vec3 uv,float biasOrLod)
{
vec2 param=texSize;
vec3 param_1=uv;
float param_2=biasOrLod;
return sc_textureLod(tex,param,param_1,param_2);
}

vec4 sc_textureLod(highp sampler2D tex,vec2 uv,float lod)
{
#if (__VERSION__>=300)
    return textureLod(tex,uv,lod);
#elif defined(FRAGMENT_SHADER)
    #if sc_CanUseTextureLod
        return texture2DLod(tex,uv,lod);
    #else
        return texture2D(tex,uv,lod);
    #endif
#elif defined(VERTEX_SHADER)
    #if sc_MaxVertexTextureImageUnits>0
        return texture2DLod(tex,uv,lod);
    #else
        return vec4(0.0);
    #endif
#endif

}

vec4 sc_textureLod(highp sampler2DArray tex,vec3 uv,float lod)
{
#if (__VERSION__>=300)
    return textureLod(tex,uv,lod);
#elif (__VERSION__==120)
    return texture2DArrayLod(tex,uv,lod);
#else
    return vec4(0.0);
#endif

}

vec4 sc_textureLod(highp sampler3D tex,vec3 uv,float lod)
{
#if (__VERSION__>=300)
    return textureLod(tex,uv,lod);
#elif (__VERSION__==120)
    return texture3DLod(tex,uv,lod);
#else
    return vec4(0.0);
#endif

}

vec4 sc_textureLod(highp samplerCube tex,vec3 uv,float lod)
{
#if __VERSION__>=300
    return textureLod(tex,uv,lod);
#elif __VERSION__==100
    return vec4(0.0);
#elif defined(FRAGMENT_SHADER)
    #if sc_CanUseTextureLod
        return textureCubeLod(tex,uv,lod);
    #else
        return textureCube(tex,uv,lod);
    #endif
#elif defined(VERTEX_SHADER)
    #if sc_MaxVertexTextureImageUnits>0
        return textureCubeLod(tex,uv,lod);
    #else
        return vec4(0.0);
    #endif
#endif

}

vec2 sc_SamplingCoordsGlobalToView(inout vec3 uvi,int renderingLayout,int viewIndex)
{
if (renderingLayout==1)
{
uvi.y=((2.0*uvi.y)+float(viewIndex))-1.0;
}
return uvi.xy;
}

vec2 sc_ScreenCoordsGlobalToView(vec2 uv)
{
#if (sc_StereoRenderingMode==1)
{
vec3 uvi=vec3(uv,0.0);
int instancedClippedLayout=1;
int viewIndex=sc_GetStereoViewIndex();
vec3 param=vec3(uv,0.0);
int param_1=instancedClippedLayout;
int param_2=viewIndex;
vec2 _337=sc_SamplingCoordsGlobalToView(param,param_1,param_2);
return _337;
}
#else
{
return uv;
}
#endif
}

vec3 sc_SamplingCoordsViewToGlobal(vec2 uv,int renderingLayout,int viewIndex)
{
if (renderingLayout==0)
{
return vec3(uv,0.0);
}
else
{
if (renderingLayout==1)
{
return vec3(uv.x,(uv.y*0.5)+(0.5-(float(viewIndex)*0.5)),0.0);
}
else
{
return vec3(uv,float(viewIndex));
}
}
}

vec4 sc_SampleViewLevel(highp sampler2D tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex,float level)
{
vec2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
vec3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
vec2 param_3=texSize;
vec2 param_4=uvi.xy;
float param_5=level;
return sc_textureLod(tex,param_3,param_4,param_5);
}

vec4 sc_SampleView(highp sampler2D tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex,float bias)
{
vec2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
vec3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
vec2 param_3=texSize;
vec2 param_4=uvi.xy;
float param_5=bias;
return sc_textureBiasOrLod(tex,param_3,param_4,param_5);
}

vec4 sc_SampleView(highp sampler2D tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex)
{
vec2 param=texSize;
vec2 param_1=uv;
int param_2=renderingLayout;
int param_3=viewIndex;
float param_4=0.0;
return sc_SampleView(tex,param,param_1,param_2,param_3,param_4);
}

vec4 sc_SampleViewLevel(highp sampler2DArray tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex,float level)
{
vec2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
vec3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
vec2 param_3=texSize;
vec3 param_4=uvi;
float param_5=level;
return sc_textureLod(tex,param_3,param_4,param_5);
}

vec4 sc_SampleView(highp sampler2DArray tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex,float bias)
{
vec2 param=uv;
int param_1=renderingLayout;
int param_2=viewIndex;
vec3 uvi=sc_SamplingCoordsViewToGlobal(param,param_1,param_2);
vec2 param_3=texSize;
vec3 param_4=uvi;
float param_5=bias;
return sc_textureBiasOrLod(tex,param_3,param_4,param_5);
}

vec4 sc_SampleView(highp sampler2DArray tex,vec2 texSize,vec2 uv,int renderingLayout,int viewIndex)
{
vec2 param=texSize;
vec2 param_1=uv;
int param_2=renderingLayout;
int param_3=viewIndex;
float param_4=0.0;
return sc_SampleView(tex,param,param_1,param_2,param_3,param_4);
}

void sc_ClampUV(inout float value,float minValue,float maxValue,bool useClampToBorder,inout float clampToBorderFactor)
{
float clampedValue=clamp(value,minValue,maxValue);
float factor=step(abs(value-clampedValue),1e-05);
clampToBorderFactor*=(factor+((1.0-float(useClampToBorder))*(1.0-factor)));
value=clampedValue;
}

vec2 sc_platformUVFlip(vec2 uv)
{
return uv;
}

void sc_SoftwareWrap(inout float uv,int softwareWrapMode,bool useClampToBorder,inout float clampToBorderFactor)
{
if ((softwareWrapMode==0)||(softwareWrapMode==3))
{
float param=uv;
float param_1=0.0;
float param_2=1.0;
bool param_3=useClampToBorder;
float param_4=clampToBorderFactor;
sc_ClampUV(param,param_1,param_2,param_3,param_4);
uv=param;
clampToBorderFactor=param_4;
}
else
{
if (softwareWrapMode==1)
{
uv=fract(uv);
}
else
{
if (softwareWrapMode==2)
{
float uvFract=fract(uv);
float uvInt=uv-uvFract;
float uvOdd=step(0.25,fract(uvInt*0.5));
uv=mix(uvFract,1.0-uvFract,clamp(uvOdd,0.0,1.0));
}
}
}
}

vec2 quantizeUV(vec2 uv)
{
return uv;
}

vec4 sc_SampleTexture(highp sampler2D sampler_,vec2 samplerDims,int renderingLayout,int viewIndex,vec2 uv,bool useUvTransform,mat3 uvTransform,ivec2 softwareWrapModes,bool useUvMinMax,vec4 uvMinMax,bool useClampToBorder,vec4 borderColor,float mipBias)
{
vec2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=vec2((uvTransform*vec3(_uv,1.0)).xy);
}
vec2 param_18=_uv;
_uv=quantizeUV(param_18);
vec2 param_19=_uv;
_uv=sc_platformUVFlip(param_19);
vec2 param_20=samplerDims;
vec2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=mipBias;
vec4 tex=sc_SampleView(sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,vec4(clampToBorderFactor));
}
return tex;
}

vec4 sc_SampleTexture(highp sampler2DArray sampler_,vec2 samplerDims,int renderingLayout,int viewIndex,vec2 uv,bool useUvTransform,mat3 uvTransform,ivec2 softwareWrapModes,bool useUvMinMax,vec4 uvMinMax,bool useClampToBorder,vec4 borderColor,float mipBias)
{
vec2 _uv=uv;
float clampToBorderFactor=1.0;
bool useClampToBorderForWrap=useClampToBorder&&(!useUvMinMax);
float param=_uv.x;
int param_1=softwareWrapModes.x;
bool param_2=useClampToBorderForWrap;
float param_3=clampToBorderFactor;
sc_SoftwareWrap(param,param_1,param_2,param_3);
_uv.x=param;
clampToBorderFactor=param_3;
float param_4=_uv.y;
int param_5=softwareWrapModes.y;
bool param_6=useClampToBorderForWrap;
float param_7=clampToBorderFactor;
sc_SoftwareWrap(param_4,param_5,param_6,param_7);
_uv.y=param_4;
clampToBorderFactor=param_7;
if (useUvMinMax)
{
float param_8=_uv.x;
float param_9=uvMinMax.x;
float param_10=uvMinMax.z;
bool param_11=useClampToBorder;
float param_12=clampToBorderFactor;
sc_ClampUV(param_8,param_9,param_10,param_11,param_12);
_uv.x=param_8;
clampToBorderFactor=param_12;
float param_13=_uv.y;
float param_14=uvMinMax.y;
float param_15=uvMinMax.w;
bool param_16=useClampToBorder;
float param_17=clampToBorderFactor;
sc_ClampUV(param_13,param_14,param_15,param_16,param_17);
_uv.y=param_13;
clampToBorderFactor=param_17;
}
if (useUvTransform)
{
_uv=vec2((uvTransform*vec3(_uv,1.0)).xy);
}
vec2 param_18=_uv;
_uv=quantizeUV(param_18);
vec2 param_19=_uv;
_uv=sc_platformUVFlip(param_19);
vec2 param_20=samplerDims;
vec2 param_21=_uv;
int param_22=renderingLayout;
int param_23=viewIndex;
float param_24=mipBias;
vec4 tex=sc_SampleView(sampler_,param_20,param_21,param_22,param_23,param_24);
if (useClampToBorder)
{
tex=mix(borderColor,tex,vec4(clampToBorderFactor));
}
return tex;
}

vec4 sampleTextureWithTransform(highp sampler2DArray samp,inout vec3 uv,bool useLOD,float lod,bool useUvTransform,mat3 uvTransform)
{
if (useUvTransform)
{
vec2 _796=vec2((uvTransform*vec3(uv.xy,1.0)).xy);
uv=vec3(_796.x,_796.y,uv.z);
}
vec2 param=uv.xy;
vec2 _802=quantizeUV(param);
uv=vec3(_802.x,_802.y,uv.z);
if (useLOD)
{
vec3 param_1=uv;
float param_2=lod;
return sc_textureLod(samp,param_1,param_2);
}
else
{
return texture2D(samp,uv);
}
}

float depthScreenToViewSpace(inout float depth,vec2 projectionMatrixTerms)
{
float m22=projectionMatrixTerms.x;
float m32=projectionMatrixTerms.y;
depth=(depth*2.0)-1.0;
return m32/((-depth)-m22);
}

float depthViewToScreenSpace(inout float depth,vec2 projectionMatrixTerms)
{
float m22=projectionMatrixTerms.x;
float m32=projectionMatrixTerms.y;
float _845;
if (depth!=0.0)
{
_845=(-m22)-(m32/depth);
}
else
{
_845=0.0;
}
depth=_845;
return (depth*0.5)+0.5;
}

vec4 sampleTextureWithTransform(highp sampler2D samp,inout vec2 uv,bool useUvTransform,mat3 uvTransform)
{
if (useUvTransform)
{
uv=vec2((uvTransform*vec3(uv,1.0)).xy);
}
vec2 param=uv;
uv=quantizeUV(param);
return texture2D(samp,uv);
}

#endif // #elif defined FRAGMENT_SHADER // #if defined VERTEX_SHADER
