#pragma once
#pragma clang diagnostic ignored "-Wmissing-prototypes"

#include <metal_stdlib>
#include <simd/simd.h>
#define STD_DISABLE_VERTEX_NORMAL 1
#define STD_DISABLE_VERTEX_TANGENT 1
#define STD_DISABLE_VERTEX_TEXTURE1 1

#include "std2_vs.metal"
#include "std2_fs.metal"

using namespace metal;

namespace SNAP_VS {


//SG_REFLECTION_BEGIN
//ubo float UserUniforms:vs21:176 {
//float4 cameraTextureSize:0
//float4 cameraTextureDims:16
//float4 cameraTextureDimsRecip:32
//float4 cameraTextureView:48
//float3x3 cameraTextureTransform:64
//float4 cameraTextureUvMinMax:112
//float4 cameraTextureBorderColor:128
//float4 leftSampleOffsetSize:144
//float4 rightSampleOffsetSize:160
//}
//SG_REFLECTION_END


struct UserUniforms
{
float4 cameraTextureSize;
float4 cameraTextureDims;
float4 cameraTextureDimsRecip;
float4 cameraTextureView;
float3x3 cameraTextureTransform;
float4 cameraTextureUvMinMax;
float4 cameraTextureBorderColor;
float4 leftSampleOffsetSize;
float4 rightSampleOffsetSize;
};

#ifndef cameraTextureHasSwappedViews
#define cameraTextureHasSwappedViews 0
#elif cameraTextureHasSwappedViews==1
#undef cameraTextureHasSwappedViews
#define cameraTextureHasSwappedViews 1
#endif
#ifndef cameraTextureLayout
#define cameraTextureLayout 0
#endif
#ifndef cameraTextureUV
#define cameraTextureUV 0
#endif
#ifndef SC_USE_UV_TRANSFORM_cameraTexture
#define SC_USE_UV_TRANSFORM_cameraTexture 0
#elif SC_USE_UV_TRANSFORM_cameraTexture==1
#undef SC_USE_UV_TRANSFORM_cameraTexture
#define SC_USE_UV_TRANSFORM_cameraTexture 1
#endif
#ifndef SC_SOFTWARE_WRAP_MODE_U_cameraTexture
#define SC_SOFTWARE_WRAP_MODE_U_cameraTexture -1
#endif
#ifndef SC_SOFTWARE_WRAP_MODE_V_cameraTexture
#define SC_SOFTWARE_WRAP_MODE_V_cameraTexture -1
#endif
#ifndef SC_USE_UV_MIN_MAX_cameraTexture
#define SC_USE_UV_MIN_MAX_cameraTexture 0
#elif SC_USE_UV_MIN_MAX_cameraTexture==1
#undef SC_USE_UV_MIN_MAX_cameraTexture
#define SC_USE_UV_MIN_MAX_cameraTexture 1
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_cameraTexture
#define SC_USE_CLAMP_TO_BORDER_cameraTexture 0
#elif SC_USE_CLAMP_TO_BORDER_cameraTexture==1
#undef SC_USE_CLAMP_TO_BORDER_cameraTexture
#define SC_USE_CLAMP_TO_BORDER_cameraTexture 1
#endif

struct main_vert_out
{
int varStereoViewID [[user(locn0)]];
float varClipDistance [[user(locn1)]];
float2 varShadowTex [[user(locn2)]];
float3 varPos [[user(locn3)]];
float3 varNormal [[user(locn4)]];
float4 varTangent [[user(locn5)]];
float4 varPackedTex [[user(locn6)]];
float4 varScreenPos [[user(locn7)]];
float2 varScreenTexturePos [[user(locn8)]];
float varViewSpaceDepth [[user(locn9)]];
float varLerp [[user(locn10)]];
float2 varLeftSampleCoord [[user(locn11)]];
float2 varRightSampleCoord [[user(locn12)]];
float4 gl_Position [[position]];
};

struct main_vert_in
{
float4 position [[attribute(0)]];
float3 normal [[attribute(1)]];
float4 tangent [[attribute(2)]];
float2 texture0 [[attribute(3)]];
float2 texture1 [[attribute(4)]];
float4 boneData [[attribute(5)]];
float3 blendShape0Pos [[attribute(6)]];
float3 blendShape1Pos [[attribute(7)]];
float3 blendShape2Pos [[attribute(8)]];
float3 blendShape3Pos [[attribute(9)]];
float3 blendShape4Pos [[attribute(10)]];
float3 blendShape5Pos [[attribute(11)]];
float3 blendShape0Normal [[attribute(12)]];
float3 blendShape1Normal [[attribute(13)]];
float3 blendShape2Normal [[attribute(14)]];
};

vertex main_vert_out main_vert(main_vert_in in [[stage_in]],constant UserUniforms& userUniformsObj [[buffer(21)]],constant LibraryUniforms& libraryUniformsObj [[buffer(22)]],uint gl_InstanceIndex [[instance_id]])
{
main_vert_out out={};
sc_Vertex_t v=sc_LoadVertexAttributes(libraryUniformsObj,in.position,in.normal,in.tangent,in.texture0,in.texture1);
float3x3 leftSampleTransform=userUniformsObj.cameraTextureTransform;
leftSampleTransform[0].x*=userUniformsObj.leftSampleOffsetSize.z;
leftSampleTransform[1].y*=userUniformsObj.leftSampleOffsetSize.w;
float3x3 rightSampleTransform=userUniformsObj.cameraTextureTransform;
rightSampleTransform[0].x*=userUniformsObj.rightSampleOffsetSize.z;
rightSampleTransform[1].y*=userUniformsObj.rightSampleOffsetSize.w;
float2 rangedTexCoord=(v.texture0*float2(2.0))-float2(1.0);
float3 leftSampleCoord=float3(rangedTexCoord,1.0)*leftSampleTransform;
float3 rightSampleCoord=float3(rangedTexCoord,1.0)*rightSampleTransform;
out.varLerp=1.0-v.texture0.x;
out.varLeftSampleCoord=((leftSampleCoord.xy+userUniformsObj.leftSampleOffsetSize.xy)*0.5)+float2(0.5);
out.varRightSampleCoord=((rightSampleCoord.xy+userUniformsObj.rightSampleOffsetSize.xy)*0.5)+float2(0.5);
sc_Vertex_t param=v;
sc_ProcessVertex(param,libraryUniformsObj,out.varClipDistance,out.varStereoViewID,out.gl_Position,in.boneData,in.blendShape0Pos,in.blendShape0Normal,in.blendShape1Pos,in.blendShape1Normal,in.blendShape2Pos,in.blendShape2Normal,in.blendShape3Pos,in.blendShape4Pos,in.blendShape5Pos,in.tangent,out.varPos,out.varNormal,out.varTangent,out.varPackedTex,out.varScreenPos,out.varScreenTexturePos,out.varShadowTex,out.varViewSpaceDepth);
return out;
}

} // VERTEX SHADER


namespace SNAP_FS {


//SG_REFLECTION_BEGIN
//ubo float UserUniforms:fs21:176 {
//float4 cameraTextureSize:0
//float4 cameraTextureDims:16
//float4 cameraTextureDimsRecip:32
//float4 cameraTextureView:48
//float3x3 cameraTextureTransform:64
//float4 cameraTextureUvMinMax:112
//float4 cameraTextureBorderColor:128
//float4 leftSampleOffsetSize:144
//float4 rightSampleOffsetSize:160
//}
//SG_REFLECTION_END


struct UserUniforms
{
float4 cameraTextureSize;
float4 cameraTextureDims;
float4 cameraTextureDimsRecip;
float4 cameraTextureView;
float3x3 cameraTextureTransform;
float4 cameraTextureUvMinMax;
float4 cameraTextureBorderColor;
float4 leftSampleOffsetSize;
float4 rightSampleOffsetSize;
};

#ifndef cameraTextureHasSwappedViews
#define cameraTextureHasSwappedViews 0
#elif cameraTextureHasSwappedViews==1
#undef cameraTextureHasSwappedViews
#define cameraTextureHasSwappedViews 1
#endif
#ifndef cameraTextureLayout
#define cameraTextureLayout 0
#endif
#ifndef SC_SOFTWARE_WRAP_MODE_U_cameraTexture
#define SC_SOFTWARE_WRAP_MODE_U_cameraTexture -1
#endif
#ifndef SC_SOFTWARE_WRAP_MODE_V_cameraTexture
#define SC_SOFTWARE_WRAP_MODE_V_cameraTexture -1
#endif
constant int2 _2048=int2(SC_SOFTWARE_WRAP_MODE_U_cameraTexture,SC_SOFTWARE_WRAP_MODE_V_cameraTexture);
#ifndef SC_USE_UV_MIN_MAX_cameraTexture
#define SC_USE_UV_MIN_MAX_cameraTexture 0
#elif SC_USE_UV_MIN_MAX_cameraTexture==1
#undef SC_USE_UV_MIN_MAX_cameraTexture
#define SC_USE_UV_MIN_MAX_cameraTexture 1
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_cameraTexture
#define SC_USE_CLAMP_TO_BORDER_cameraTexture 0
#elif SC_USE_CLAMP_TO_BORDER_cameraTexture==1
#undef SC_USE_CLAMP_TO_BORDER_cameraTexture
#define SC_USE_CLAMP_TO_BORDER_cameraTexture 1
#endif
constant int2 _2074=int2(SC_SOFTWARE_WRAP_MODE_U_cameraTexture,SC_SOFTWARE_WRAP_MODE_V_cameraTexture);
constant int2 _2103=int2(SC_SOFTWARE_WRAP_MODE_U_cameraTexture,SC_SOFTWARE_WRAP_MODE_V_cameraTexture);
constant int2 _2125=int2(SC_SOFTWARE_WRAP_MODE_U_cameraTexture,SC_SOFTWARE_WRAP_MODE_V_cameraTexture);
#ifndef cameraTextureUV
#define cameraTextureUV 0
#endif
#ifndef SC_USE_UV_TRANSFORM_cameraTexture
#define SC_USE_UV_TRANSFORM_cameraTexture 0
#elif SC_USE_UV_TRANSFORM_cameraTexture==1
#undef SC_USE_UV_TRANSFORM_cameraTexture
#define SC_USE_UV_TRANSFORM_cameraTexture 1
#endif

struct main_frag_out
{
float4 FragColor_0 [[color(0)]];
float4 FragColor_1 [[color(1)]];
float4 FragColor_2 [[color(2)]];
float4 FragColor_3 [[color(3)]];
};

struct main_frag_in
{
int varStereoViewID [[user(locn0)]];
float varLerp [[user(locn10)]];
float2 varLeftSampleCoord [[user(locn11)]];
float2 varRightSampleCoord [[user(locn12)]];
};

float2 cameraTextureGetDims2D(constant UserUniforms& userUniformsObj)
{
return userUniformsObj.cameraTextureDims.xy;
}

int cameraTextureGetStereoViewIndex(thread int& varStereoViewID)
{
#if (cameraTextureHasSwappedViews)
{
return 1-sc_GetStereoViewIndex(varStereoViewID);
}
#else
{
return sc_GetStereoViewIndex(varStereoViewID);
}
#endif
}

fragment main_frag_out main_frag(main_frag_in in [[stage_in]],constant UserUniforms& userUniformsObj [[buffer(21)]],texture2d<float> cameraTexture [[texture(0)]],texture2d_array<float> cameraTextureArrSC [[texture(1)]],sampler cameraTextureSmplr [[sampler(0)]],sampler cameraTextureArrSCSmplr [[sampler(1)]],float4 gl_FragCoord [[position]])
{
main_frag_out out={};
float4 FragColor[4]={};
float3 leftSample=float3(0.0);
float4 cameraTextureSample;
#if (cameraTextureLayout==2)
{
float2 param=cameraTextureGetDims2D(userUniformsObj);
int param_1=cameraTextureLayout;
int param_2=cameraTextureGetStereoViewIndex(in.varStereoViewID);
float2 param_3=in.varLeftSampleCoord;
bool param_4=false;
float3x3 param_5=float3x3(float3(1.0,0.0,0.0),float3(0.0,1.0,0.0),float3(0.0,0.0,1.0));
int2 param_6=_2048;
bool param_7=(int(SC_USE_UV_MIN_MAX_cameraTexture)!=0);
float4 param_8=userUniformsObj.cameraTextureUvMinMax;
bool param_9=(int(SC_USE_CLAMP_TO_BORDER_cameraTexture)!=0);
float4 param_10=userUniformsObj.cameraTextureBorderColor;
float param_11=0.0;
cameraTextureSample=sc_SampleTexture(cameraTextureArrSC,cameraTextureArrSCSmplr,param,param_1,param_2,param_3,param_4,param_5,param_6,param_7,param_8,param_9,param_10,param_11);
}
#else
{
float2 param_12=cameraTextureGetDims2D(userUniformsObj);
int param_13=cameraTextureLayout;
int param_14=cameraTextureGetStereoViewIndex(in.varStereoViewID);
float2 param_15=in.varLeftSampleCoord;
bool param_16=false;
float3x3 param_17=float3x3(float3(1.0,0.0,0.0),float3(0.0,1.0,0.0),float3(0.0,0.0,1.0));
int2 param_18=_2074;
bool param_19=(int(SC_USE_UV_MIN_MAX_cameraTexture)!=0);
float4 param_20=userUniformsObj.cameraTextureUvMinMax;
bool param_21=(int(SC_USE_CLAMP_TO_BORDER_cameraTexture)!=0);
float4 param_22=userUniformsObj.cameraTextureBorderColor;
float param_23=0.0;
cameraTextureSample=sc_SampleTexture(cameraTexture,cameraTextureSmplr,param_12,param_13,param_14,param_15,param_16,param_17,param_18,param_19,param_20,param_21,param_22,param_23);
}
#endif
leftSample=cameraTextureSample.xyz;
float3 rightSample=float3(0.0);
float4 cameraTextureSample_1;
#if (cameraTextureLayout==2)
{
float2 param_24=cameraTextureGetDims2D(userUniformsObj);
int param_25=cameraTextureLayout;
int param_26=cameraTextureGetStereoViewIndex(in.varStereoViewID);
float2 param_27=in.varRightSampleCoord;
bool param_28=false;
float3x3 param_29=float3x3(float3(1.0,0.0,0.0),float3(0.0,1.0,0.0),float3(0.0,0.0,1.0));
int2 param_30=_2103;
bool param_31=(int(SC_USE_UV_MIN_MAX_cameraTexture)!=0);
float4 param_32=userUniformsObj.cameraTextureUvMinMax;
bool param_33=(int(SC_USE_CLAMP_TO_BORDER_cameraTexture)!=0);
float4 param_34=userUniformsObj.cameraTextureBorderColor;
float param_35=0.0;
cameraTextureSample_1=sc_SampleTexture(cameraTextureArrSC,cameraTextureArrSCSmplr,param_24,param_25,param_26,param_27,param_28,param_29,param_30,param_31,param_32,param_33,param_34,param_35);
}
#else
{
float2 param_36=cameraTextureGetDims2D(userUniformsObj);
int param_37=cameraTextureLayout;
int param_38=cameraTextureGetStereoViewIndex(in.varStereoViewID);
float2 param_39=in.varRightSampleCoord;
bool param_40=false;
float3x3 param_41=float3x3(float3(1.0,0.0,0.0),float3(0.0,1.0,0.0),float3(0.0,0.0,1.0));
int2 param_42=_2125;
bool param_43=(int(SC_USE_UV_MIN_MAX_cameraTexture)!=0);
float4 param_44=userUniformsObj.cameraTextureUvMinMax;
bool param_45=(int(SC_USE_CLAMP_TO_BORDER_cameraTexture)!=0);
float4 param_46=userUniformsObj.cameraTextureBorderColor;
float param_47=0.0;
cameraTextureSample_1=sc_SampleTexture(cameraTexture,cameraTextureSmplr,param_36,param_37,param_38,param_39,param_40,param_41,param_42,param_43,param_44,param_45,param_46,param_47);
}
#endif
rightSample=cameraTextureSample_1.xyz;
float3 lerpedColor=mix(leftSample,rightSample,float3(in.varLerp));
float4 param_48=float4(lerpedColor,1.0);
sc_writeFragData0(param_48,FragColor);
out.FragColor_0=FragColor[0];
out.FragColor_1=FragColor[1];
out.FragColor_2=FragColor[2];
out.FragColor_3=FragColor[3];
return out;
}

} // FRAGMENT SHADER
