#pragma once
#pragma clang diagnostic ignored "-Wmissing-prototypes"

#include <metal_stdlib>
#include <simd/simd.h>
#define STD_DISABLE_VERTEX_NORMAL 1
#define STD_DISABLE_VERTEX_TANGENT 1
#define STD_DISABLE_VERTEX_TEXTURE1 1

#include "required2.metal"
#include "std2_vs.metal"
#include "std2_fs.metal"
#include "std2_texture.metal"

using namespace metal;

namespace SNAP_VS {


//SG_REFLECTION_BEGIN
//attribute vec4 atbCoord2d:15:
//ubo float UserUniforms:vs21:960 {
//float4x4 ptsInvMat:0:[10]:64
//float4 coeffs:640:[10]:16
//float3 camDirO:800:[10]:16
//}
//SG_REFLECTION_END


struct UserUniforms
{
float4x4 ptsInvMat[10];
float4 coeffs[10];
float3 camDirO[10];
};

#ifndef CAMERA_ORTHO
#define CAMERA_ORTHO 0
#elif CAMERA_ORTHO==1
#undef CAMERA_ORTHO
#define CAMERA_ORTHO 1
#endif
#ifndef LIQUIFY_FACESTRETCH
#define LIQUIFY_FACESTRETCH 0
#elif LIQUIFY_FACESTRETCH==1
#undef LIQUIFY_FACESTRETCH
#define LIQUIFY_FACESTRETCH 1
#endif
#ifndef MAX_LIQUIFY
#define MAX_LIQUIFY 10
#endif

struct main_vert_out
{
int varStereoViewID [[user(locn0)]];
float varClipDistance [[user(locn1)]];
float2 varShadowTex [[user(locn2)]];
float3 varPos [[user(locn3)]];
float3 varNormal [[user(locn4)]];
float4 varTangent [[user(locn5)]];
float4 varPackedTex [[user(locn6)]];
float4 varScreenPos [[user(locn7)]];
float2 varScreenTexturePos [[user(locn8)]];
float varViewSpaceDepth [[user(locn9)]];
float2 varScreenSpacePointsPos0 [[user(locn10)]];
float2 varScreenSpacePointsPos1 [[user(locn11)]];
float2 varScreenSpacePointsPos2 [[user(locn12)]];
float2 varScreenSpacePointsPos3 [[user(locn13)]];
float2 varScreenSpacePointsPos4 [[user(locn14)]];
float2 varScreenSpacePointsPos5 [[user(locn15)]];
float2 varScreenSpacePointsPos6 [[user(locn16)]];
float2 varScreenSpacePointsPos7 [[user(locn17)]];
float2 varScreenSpacePointsPos8 [[user(locn18)]];
float2 varScreenSpacePointsPos9 [[user(locn19)]];
float4 gl_Position [[position]];
};

struct main_vert_in
{
float4 position [[attribute(0)]];
float3 normal [[attribute(1)]];
float4 tangent [[attribute(2)]];
float2 texture0 [[attribute(3)]];
float2 texture1 [[attribute(4)]];
float4 boneData [[attribute(5)]];
float3 blendShape0Pos [[attribute(6)]];
float3 blendShape1Pos [[attribute(7)]];
float3 blendShape2Pos [[attribute(8)]];
float3 blendShape3Pos [[attribute(9)]];
float3 blendShape4Pos [[attribute(10)]];
float3 blendShape5Pos [[attribute(11)]];
float3 blendShape0Normal [[attribute(12)]];
float3 blendShape1Normal [[attribute(13)]];
float3 blendShape2Normal [[attribute(14)]];
float4 atbCoord2d [[attribute(15)]];
};

float2 calculateObjSpace(thread const int& i,thread const float4& worldPos,constant LibraryUniforms& libraryUniformsObj,constant UserUniforms& userUniformsObj)
{
float4 vertexPosObjSpace=userUniformsObj.ptsInvMat[i]*worldPos;
float3 viewDirObjSpace;
float4 camPosObjSpace;
#if (CAMERA_ORTHO)
{
viewDirObjSpace=userUniformsObj.camDirO[i];
camPosObjSpace=vertexPosObjSpace-float4(viewDirObjSpace,0.0);
}
#else
{
camPosObjSpace=userUniformsObj.ptsInvMat[i]*float4(libraryUniformsObj.sc_Camera.position,1.0);
viewDirObjSpace=normalize(vertexPosObjSpace.xyz-camPosObjSpace.xyz);
}
#endif
float denom=viewDirObjSpace.z;
float t=10000.0;
if (denom!=0.0)
{
t=(-camPosObjSpace.z)/denom;
}
if (t<0.0)
{
t=10000.0;
}
return camPosObjSpace.xy+(viewDirObjSpace.xy*t);
}

vertex main_vert_out main_vert(main_vert_in in [[stage_in]],constant UserUniforms& userUniformsObj [[buffer(21)]],constant LibraryUniforms& libraryUniformsObj [[buffer(22)]],uint gl_InstanceIndex [[instance_id]])
{
main_vert_out out={};
sc_Vertex_t v=sc_LoadVertexAttributes(libraryUniformsObj,in.position,in.normal,in.tangent,in.texture0,in.texture1);
v.position=float4(in.atbCoord2d.xy,0.0,1.0);
sc_Vertex_t param=v;
sc_ProcessVertex(param,libraryUniformsObj,out.varClipDistance,out.varStereoViewID,out.gl_Position,in.boneData,in.blendShape0Pos,in.blendShape0Normal,in.blendShape1Pos,in.blendShape1Normal,in.blendShape2Pos,in.blendShape2Normal,in.blendShape3Pos,in.blendShape4Pos,in.blendShape5Pos,in.tangent,out.varPos,out.varNormal,out.varTangent,out.varPackedTex,out.varScreenPos,out.varScreenTexturePos,out.varShadowTex,out.varViewSpaceDepth);
out.varScreenPos=in.atbCoord2d;
float4 screenPos=float4(in.atbCoord2d.xy,-1.0,1.0);
float2 floatSum=float2(in.atbCoord2d.z,in.atbCoord2d.w);
#if (LIQUIFY_FACESTRETCH)
{
float2 _2243=out.varScreenPos.xy+floatSum;
out.varScreenPos=float4(_2243.x,_2243.y,out.varScreenPos.z,out.varScreenPos.w);
float2 _2249=screenPos.xy+floatSum;
screenPos=float4(_2249.x,_2249.y,screenPos.z,screenPos.w);
}
#endif
float4 worldPos=libraryUniformsObj.sc_ViewProjectionMatrixInverseArray[sc_GetStereoViewIndex()]*screenPos;
float3 _2264=worldPos.xyz/float3(worldPos.w);
worldPos=float4(_2264.x,_2264.y,_2264.z,worldPos.w);
worldPos.w=1.0;
#if (MAX_LIQUIFY>0)
{
int param_1=0;
float4 param_2=worldPos;
out.varScreenSpacePointsPos0=calculateObjSpace(param_1,param_2,libraryUniformsObj,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>1)
{
int param_3=1;
float4 param_4=worldPos;
out.varScreenSpacePointsPos1=calculateObjSpace(param_3,param_4,libraryUniformsObj,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>2)
{
int param_5=2;
float4 param_6=worldPos;
out.varScreenSpacePointsPos2=calculateObjSpace(param_5,param_6,libraryUniformsObj,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>3)
{
int param_7=3;
float4 param_8=worldPos;
out.varScreenSpacePointsPos3=calculateObjSpace(param_7,param_8,libraryUniformsObj,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>4)
{
int param_9=4;
float4 param_10=worldPos;
out.varScreenSpacePointsPos4=calculateObjSpace(param_9,param_10,libraryUniformsObj,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>5)
{
int param_11=5;
float4 param_12=worldPos;
out.varScreenSpacePointsPos5=calculateObjSpace(param_11,param_12,libraryUniformsObj,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>6)
{
int param_13=6;
float4 param_14=worldPos;
out.varScreenSpacePointsPos6=calculateObjSpace(param_13,param_14,libraryUniformsObj,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>7)
{
int param_15=7;
float4 param_16=worldPos;
out.varScreenSpacePointsPos7=calculateObjSpace(param_15,param_16,libraryUniformsObj,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>8)
{
int param_17=8;
float4 param_18=worldPos;
out.varScreenSpacePointsPos8=calculateObjSpace(param_17,param_18,libraryUniformsObj,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>9)
{
int param_19=9;
float4 param_20=worldPos;
out.varScreenSpacePointsPos9=calculateObjSpace(param_19,param_20,libraryUniformsObj,userUniformsObj);
}
#endif
#if (!LIQUIFY_FACESTRETCH)
{
float2 _2358=floatSum*0.5;
out.varScreenPos=float4(out.varScreenPos.x,out.varScreenPos.y,_2358.x,_2358.y);
}
#endif
out.varScreenPos.w=-out.varScreenPos.w;
return out;
}

} // VERTEX SHADER


namespace SNAP_FS {


//SG_REFLECTION_BEGIN
//ubo float UserUniforms:fs21:960 {
//float4x4 ptsInvMat:0:[10]:64
//float4 coeffs:640:[10]:16
//float3 camDirO:800:[10]:16
//}
//SG_REFLECTION_END


struct UserUniforms
{
float4x4 ptsInvMat[10];
float4 coeffs[10];
float3 camDirO[10];
};

#ifndef MAX_LIQUIFY
#define MAX_LIQUIFY 10
#endif
#ifndef LIQUIFY_FACESTRETCH
#define LIQUIFY_FACESTRETCH 0
#elif LIQUIFY_FACESTRETCH==1
#undef LIQUIFY_FACESTRETCH
#define LIQUIFY_FACESTRETCH 1
#endif
#ifndef CAMERA_ORTHO
#define CAMERA_ORTHO 0
#elif CAMERA_ORTHO==1
#undef CAMERA_ORTHO
#define CAMERA_ORTHO 1
#endif

struct main_frag_out
{
float4 FragColor_0 [[color(0)]];
float4 FragColor_1 [[color(1)]];
float4 FragColor_2 [[color(2)]];
float4 FragColor_3 [[color(3)]];
};

struct main_frag_in
{
int varStereoViewID [[user(locn0)]];
float4 varScreenPos [[user(locn7)]];
float2 varScreenSpacePointsPos0 [[user(locn10)]];
float2 varScreenSpacePointsPos1 [[user(locn11)]];
float2 varScreenSpacePointsPos2 [[user(locn12)]];
float2 varScreenSpacePointsPos3 [[user(locn13)]];
float2 varScreenSpacePointsPos4 [[user(locn14)]];
float2 varScreenSpacePointsPos5 [[user(locn15)]];
float2 varScreenSpacePointsPos6 [[user(locn16)]];
float2 varScreenSpacePointsPos7 [[user(locn17)]];
float2 varScreenSpacePointsPos8 [[user(locn18)]];
float2 varScreenSpacePointsPos9 [[user(locn19)]];
};

float liquifyVector(thread const float& curDistanceSquared,thread const float& radiusSquared,thread const float& coeficient)
{
return pow(fast::clamp(curDistanceSquared/radiusSquared,0.00078125,1.0),coeficient);
}

float2 calcOffsetForPoint(thread const int& i,thread const float2& varScreenSpacePointsPos,thread const float4& varScreenPos,constant UserUniforms& userUniformsObj)
{
float dist=dot(varScreenSpacePointsPos,varScreenSpacePointsPos);
float2 fromPointCenterVector=varScreenPos.xy-userUniformsObj.coeffs[i].xy;
float param=dist;
float param_1=userUniformsObj.coeffs[i].w;
float param_2=userUniformsObj.coeffs[i].z;
float vecCoef=liquifyVector(param,param_1,param_2)-1.0;
vecCoef*=step(dist,userUniformsObj.coeffs[i].w);
return fromPointCenterVector*vecCoef;
}

fragment main_frag_out main_frag(main_frag_in in [[stage_in]],constant UserUniforms& userUniformsObj [[buffer(21)]],constant LibraryUniforms& libraryUniformsObj [[buffer(22)]],texture2d<float> sc_ScreenTexture [[texture(0)]],texture2d_array<float> sc_ScreenTextureArrSC [[texture(1)]],sampler sc_ScreenTextureSmplr [[sampler(0)]],sampler sc_ScreenTextureArrSCSmplr [[sampler(1)]],float4 gl_FragCoord [[position]])
{
main_frag_out out={};
float4 FragColor[4]={};
float2 liqVec=float2(0.0);
float2 posToFetch=in.varScreenPos.xy;
#if (MAX_LIQUIFY>0)
{
int param=0;
float2 param_1=in.varScreenSpacePointsPos0;
float4 param_2=in.varScreenPos;
liqVec+=calcOffsetForPoint(param,param_1,param_2,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>1)
{
int param_3=1;
float2 param_4=in.varScreenSpacePointsPos1;
float4 param_5=in.varScreenPos;
liqVec+=calcOffsetForPoint(param_3,param_4,param_5,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>2)
{
int param_6=2;
float2 param_7=in.varScreenSpacePointsPos2;
float4 param_8=in.varScreenPos;
liqVec+=calcOffsetForPoint(param_6,param_7,param_8,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>3)
{
int param_9=3;
float2 param_10=in.varScreenSpacePointsPos3;
float4 param_11=in.varScreenPos;
liqVec+=calcOffsetForPoint(param_9,param_10,param_11,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>4)
{
int param_12=4;
float2 param_13=in.varScreenSpacePointsPos4;
float4 param_14=in.varScreenPos;
liqVec+=calcOffsetForPoint(param_12,param_13,param_14,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>5)
{
int param_15=5;
float2 param_16=in.varScreenSpacePointsPos5;
float4 param_17=in.varScreenPos;
liqVec+=calcOffsetForPoint(param_15,param_16,param_17,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>6)
{
int param_18=6;
float2 param_19=in.varScreenSpacePointsPos6;
float4 param_20=in.varScreenPos;
liqVec+=calcOffsetForPoint(param_18,param_19,param_20,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>7)
{
int param_21=7;
float2 param_22=in.varScreenSpacePointsPos7;
float4 param_23=in.varScreenPos;
liqVec+=calcOffsetForPoint(param_21,param_22,param_23,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>8)
{
int param_24=8;
float2 param_25=in.varScreenSpacePointsPos8;
float4 param_26=in.varScreenPos;
liqVec+=calcOffsetForPoint(param_24,param_25,param_26,userUniformsObj);
}
#endif
#if (MAX_LIQUIFY>9)
{
int param_27=9;
float2 param_28=in.varScreenSpacePointsPos9;
float4 param_29=in.varScreenPos;
liqVec+=calcOffsetForPoint(param_27,param_28,param_29,userUniformsObj);
}
#endif
posToFetch+=liqVec;
posToFetch=(posToFetch*float2(0.5,-0.5))+float2(0.5);
#if (!LIQUIFY_FACESTRETCH)
{
posToFetch+=in.varScreenPos.zw;
}
#endif
float2 param_30=posToFetch;
float4 resultColor=sc_ScreenTextureSampleView(param_30,in.varStereoViewID,libraryUniformsObj,sc_ScreenTextureArrSC,sc_ScreenTextureArrSCSmplr,sc_ScreenTexture,sc_ScreenTextureSmplr);
float4 param_31=resultColor;
sc_writeFragData0(param_31,FragColor);
out.FragColor_0=FragColor[0];
out.FragColor_1=FragColor[1];
out.FragColor_2=FragColor[2];
out.FragColor_3=FragColor[3];
return out;
}

} // FRAGMENT SHADER
