#pragma once
#pragma clang diagnostic ignored "-Wmissing-prototypes"

#include <metal_stdlib>
#include <simd/simd.h>
#define STD_DISABLE_VERTEX_NORMAL 1
#define STD_DISABLE_VERTEX_TANGENT 1
#define STD_DISABLE_VERTEX_TEXTURE1 1

#include "std2_vs.metal"
#include "std2_fs.metal"
#include "std2_texture.metal"

using namespace metal;

namespace SNAP_VS {


//SG_REFLECTION_BEGIN
//ubo float UserUniforms:vs21:160 {
//float2 screenTextureProjectionMatrixTerms:0
//float4 screenTextureUvMinMax:16
//float4 screenTextureSize:32
//float3x3 screenTextureTransform:48
//float2 currentProjectionMatrixTerms:96
//float2 inputProjectionMatrixTerms:104
//float3x3 meshTransform:112
//}
//SG_REFLECTION_END

#ifndef USE_MESH_TRANSFORM
#define USE_MESH_TRANSFORM 0
#elif USE_MESH_TRANSFORM==1
#undef USE_MESH_TRANSFORM
#define USE_MESH_TRANSFORM 1
#endif

struct UserUniforms
{
float2 screenTextureProjectionMatrixTerms;
float4 screenTextureUvMinMax;
float4 screenTextureSize;
float3x3 screenTextureTransform;
float2 currentProjectionMatrixTerms;
float2 inputProjectionMatrixTerms;
float3x3 meshTransform;
};

#ifndef ONLY_VERTEX_ATTRIBUTE
#define ONLY_VERTEX_ATTRIBUTE 0
#elif ONLY_VERTEX_ATTRIBUTE==1
#undef ONLY_VERTEX_ATTRIBUTE
#define ONLY_VERTEX_ATTRIBUTE 1
#endif
#ifndef SC_USE_UV_TRANSFORM_screenTexture
#define SC_USE_UV_TRANSFORM_screenTexture 0
#elif SC_USE_UV_TRANSFORM_screenTexture==1
#undef SC_USE_UV_TRANSFORM_screenTexture
#define SC_USE_UV_TRANSFORM_screenTexture 1
#endif

struct main_vert_out
{
int varStereoViewID [[user(locn0)]];
float varClipDistance [[user(locn1)]];
float2 varShadowTex [[user(locn2)]];
float3 varPos [[user(locn3)]];
float3 varNormal [[user(locn4)]];
float4 varTangent [[user(locn5)]];
float4 varPackedTex [[user(locn6)]];
float4 varScreenPos [[user(locn7)]];
float2 varScreenTexturePos [[user(locn8)]];
float varViewSpaceDepth [[user(locn9)]];
float4 gl_Position [[position]];
};

struct main_vert_in
{
float4 position [[attribute(0)]];
float3 normal [[attribute(1)]];
float4 tangent [[attribute(2)]];
float2 texture0 [[attribute(3)]];
float2 texture1 [[attribute(4)]];
float4 boneData [[attribute(5)]];
float3 blendShape0Pos [[attribute(6)]];
float3 blendShape1Pos [[attribute(7)]];
float3 blendShape2Pos [[attribute(8)]];
float3 blendShape3Pos [[attribute(9)]];
float3 blendShape4Pos [[attribute(10)]];
float3 blendShape5Pos [[attribute(11)]];
float3 blendShape0Normal [[attribute(12)]];
float3 blendShape1Normal [[attribute(13)]];
float3 blendShape2Normal [[attribute(14)]];
};

vertex main_vert_out main_vert(main_vert_in in [[stage_in]],constant UserUniforms& userUniformsObj [[buffer(21)]],constant LibraryUniforms& libraryUniformsObj [[buffer(22)]],uint gl_InstanceIndex [[instance_id]])
{
main_vert_out out={};
sc_Vertex_t v=sc_LoadVertexAttributes(libraryUniformsObj,in.position,in.normal,in.tangent,in.texture0,in.texture1);
float4 unmodifiedPosition=v.position;
#if (USE_MESH_TRANSFORM)
{
v.position=float4(userUniformsObj.meshTransform*float3(v.position.xy,1.0),1.0);
}
#endif
sc_Vertex_t param=v;
sc_ProcessVertex(param,libraryUniformsObj,out.varClipDistance,out.varStereoViewID,out.gl_Position,in.boneData,in.blendShape0Pos,in.blendShape0Normal,in.blendShape1Pos,in.blendShape1Normal,in.blendShape2Pos,in.blendShape2Normal,in.blendShape3Pos,in.blendShape4Pos,in.blendShape5Pos,in.tangent,out.varPos,out.varNormal,out.varTangent,out.varPackedTex,out.varScreenPos,out.varScreenTexturePos,out.varShadowTex,out.varViewSpaceDepth);
#if (ONLY_VERTEX_ATTRIBUTE)
{
float2 _2169=(unmodifiedPosition.xy+float2(1.0))*0.5;
out.varPackedTex=float4(_2169.x,_2169.y,out.varPackedTex.z,out.varPackedTex.w);
}
#endif
return out;
}

} // VERTEX SHADER


namespace SNAP_FS {


//SG_REFLECTION_BEGIN
//ubo float UserUniforms:fs21:160 {
//float2 screenTextureProjectionMatrixTerms:0
//float4 screenTextureUvMinMax:16
//float4 screenTextureSize:32
//float3x3 screenTextureTransform:48
//float2 currentProjectionMatrixTerms:96
//float2 inputProjectionMatrixTerms:104
//float3x3 meshTransform:112
//}
//SG_REFLECTION_END

#ifndef SC_USE_UV_TRANSFORM_screenTexture
#define SC_USE_UV_TRANSFORM_screenTexture 0
#elif SC_USE_UV_TRANSFORM_screenTexture==1
#undef SC_USE_UV_TRANSFORM_screenTexture
#define SC_USE_UV_TRANSFORM_screenTexture 1
#endif

struct UserUniforms
{
float2 screenTextureProjectionMatrixTerms;
float4 screenTextureUvMinMax;
float4 screenTextureSize;
float3x3 screenTextureTransform;
float2 currentProjectionMatrixTerms;
float2 inputProjectionMatrixTerms;
float3x3 meshTransform;
};

#ifndef USE_MESH_TRANSFORM
#define USE_MESH_TRANSFORM 0
#elif USE_MESH_TRANSFORM==1
#undef USE_MESH_TRANSFORM
#define USE_MESH_TRANSFORM 1
#endif
#ifndef ONLY_VERTEX_ATTRIBUTE
#define ONLY_VERTEX_ATTRIBUTE 0
#elif ONLY_VERTEX_ATTRIBUTE==1
#undef ONLY_VERTEX_ATTRIBUTE
#define ONLY_VERTEX_ATTRIBUTE 1
#endif

struct main_frag_out
{
float gl_FragDepth [[depth(any)]];
};

struct main_frag_in
{
float4 varPackedTex [[user(locn6)]];
};

fragment main_frag_out main_frag(main_frag_in in [[stage_in]],constant UserUniforms& userUniformsObj [[buffer(21)]],texture2d<float> screenTexture [[texture(0)]],sampler screenTextureSmplr [[sampler(0)]],float4 gl_FragCoord [[position]])
{
main_frag_out out={};
float2 param=in.varPackedTex.xy;
bool param_1=(int(SC_USE_UV_TRANSFORM_screenTexture)!=0);
float3x3 param_2=userUniformsObj.screenTextureTransform;
float4 _1844=sampleTextureWithTransform(screenTexture,screenTextureSmplr,param,param_1,param_2);
float depth=_1844.x;
if (depth<1.0)
{
float param_3=depth;
float2 param_4=userUniformsObj.inputProjectionMatrixTerms;
float _1857=depthScreenToViewSpace(param_3,param_4);
depth=_1857;
float param_5=depth;
float2 param_6=userUniformsObj.currentProjectionMatrixTerms;
float _1864=depthViewToScreenSpace(param_5,param_6);
depth=_1864;
}
out.gl_FragDepth=depth;
return out;
}

} // FRAGMENT SHADER
