#pragma once
#pragma clang diagnostic ignored "-Wmissing-prototypes"

#include <metal_stdlib>
#include <simd/simd.h>
#define STD_DISABLE_VERTEX_NORMAL 1
#define STD_DISABLE_VERTEX_TANGENT 1
#define STD_DISABLE_VERTEX_TEXTURE1 1

#include "std2_vs.metal"
#include "std2_fs.metal"

using namespace metal;

namespace SNAP_VS {


//SG_REFLECTION_BEGIN
//ubo float UserUniforms:vs21:16 {
//float3 uniMainTextureSize:0
//}
//SG_REFLECTION_END


struct UserUniforms
{
float3 uniMainTextureSize;
};

struct main_vert_out
{
int varStereoViewID [[user(locn0)]];
float varClipDistance [[user(locn1)]];
float2 varShadowTex [[user(locn2)]];
float3 varPos [[user(locn3)]];
float3 varNormal [[user(locn4)]];
float4 varTangent [[user(locn5)]];
float4 varPackedTex [[user(locn6)]];
float4 varScreenPos [[user(locn7)]];
float2 varScreenTexturePos [[user(locn8)]];
float varViewSpaceDepth [[user(locn9)]];
float4 gl_Position [[position]];
};

struct main_vert_in
{
float4 position [[attribute(0)]];
float3 normal [[attribute(1)]];
float4 tangent [[attribute(2)]];
float2 texture0 [[attribute(3)]];
float2 texture1 [[attribute(4)]];
float4 boneData [[attribute(5)]];
float3 blendShape0Pos [[attribute(6)]];
float3 blendShape1Pos [[attribute(7)]];
float3 blendShape2Pos [[attribute(8)]];
float3 blendShape3Pos [[attribute(9)]];
float3 blendShape4Pos [[attribute(10)]];
float3 blendShape5Pos [[attribute(11)]];
float3 blendShape0Normal [[attribute(12)]];
float3 blendShape1Normal [[attribute(13)]];
float3 blendShape2Normal [[attribute(14)]];
};

vertex main_vert_out main_vert(main_vert_in in [[stage_in]],constant LibraryUniforms& libraryUniformsObj [[buffer(22)]],uint gl_InstanceIndex [[instance_id]])
{
main_vert_out out={};
sc_Vertex_t v=sc_LoadVertexAttributes(libraryUniformsObj,in.position,in.normal,in.tangent,in.texture0,in.texture1);
sc_Vertex_t param=v;
sc_ProcessVertex(param,libraryUniformsObj,out.varClipDistance,out.varStereoViewID,out.gl_Position,in.boneData,in.blendShape0Pos,in.blendShape0Normal,in.blendShape1Pos,in.blendShape1Normal,in.blendShape2Pos,in.blendShape2Normal,in.blendShape3Pos,in.blendShape4Pos,in.blendShape5Pos,in.tangent,out.varPos,out.varNormal,out.varTangent,out.varPackedTex,out.varScreenPos,out.varScreenTexturePos,out.varShadowTex,out.varViewSpaceDepth);
return out;
}

} // VERTEX SHADER


namespace SNAP_FS {


//SG_REFLECTION_BEGIN
//ubo float UserUniforms:fs21:16 {
//float3 uniMainTextureSize:0
//}
//SG_REFLECTION_END


struct UserUniforms
{
float3 uniMainTextureSize;
};

struct main_frag_out
{
float4 FragColor_0 [[color(0)]];
float4 FragColor_1 [[color(1)]];
float4 FragColor_2 [[color(2)]];
float4 FragColor_3 [[color(3)]];
};

struct main_frag_in
{
float4 varPackedTex [[user(locn6)]];
};

fragment main_frag_out main_frag(main_frag_in in [[stage_in]],constant UserUniforms& userUniformsObj [[buffer(21)]],float4 gl_FragCoord [[position]])
{
main_frag_out out={};
float4 FragColor[4]={};
float4 pixel=float4(0.0,0.0,1.0,1.0);
float2 uvAbsolute=in.varPackedTex.xy*userUniformsObj.uniMainTextureSize.xy;
float2 uvdx=dfdx(uvAbsolute);
float2 uvdy=dfdy(uvAbsolute);
float uLength2=dot(uvdx,uvdx);
float vLength2=dot(uvdy,uvdy);
float lengthMax=fast::max(uLength2,vLength2);
bool baseLevelPass=false;
float mipLevel=(0.5*log2(lengthMax))-0.5;
if (userUniformsObj.uniMainTextureSize.z==1.0)
{
baseLevelPass=mipLevel<0.0;
}
else
{
float mipThreshold=0.1;
baseLevelPass=mipLevel<mipThreshold;
}
if (baseLevelPass)
{
pixel=float4(float3(0.0,1.0,0.0).x,float3(0.0,1.0,0.0).y,float3(0.0,1.0,0.0).z,pixel.w);
}
else
{
pixel=float4(float3(1.0,0.0,0.0).x,float3(1.0,0.0,0.0).y,float3(1.0,0.0,0.0).z,pixel.w);
}
float4 param=pixel;
sc_writeFragData0(param,FragColor);
out.FragColor_0=FragColor[0];
out.FragColor_1=FragColor[1];
out.FragColor_2=FragColor[2];
out.FragColor_3=FragColor[3];
return out;
}

} // FRAGMENT SHADER
