#pragma once
#include <metal_stdlib>
#include <simd/simd.h>
#include "std2.metal"

using namespace metal;

namespace SNAP_VS {


//SG_REFLECTION_BEGIN
//SG_REFLECTION_END


float2 getProjectedTexCoords(thread const float4& vertexPosition,constant LibraryUniforms& libraryUniformsObj)
{
float4 worldPos=vertexPosition;
#if (sc_RenderingSpace==1)
{
worldPos=libraryUniformsObj.sc_ModelMatrix*vertexPosition;
}
#endif
float4 projectedUVs=libraryUniformsObj.sc_ProjectorMatrix*worldPos;
return ((projectedUVs.xy/float2(projectedUVs.w))*0.5)+float2(0.5);
}

} // VERTEX SHADER


namespace SNAP_FS {


//SG_REFLECTION_BEGIN
//SG_REFLECTION_END


float getShadowAlpha(thread const float4& color)
{
float shadowAlpha=1.0;
#if ((sc_BlendMode_Normal||sc_BlendMode_AlphaToCoverage)||sc_BlendMode_PremultipliedAlphaHardware)
{
shadowAlpha=color.w;
}
#else
{
#if (sc_BlendMode_PremultipliedAlpha)
{
shadowAlpha=fast::clamp(color.w*2.0,0.0,1.0);
}
#else
{
#if (sc_BlendMode_AddWithAlphaFactor)
{
shadowAlpha=fast::clamp(dot(color.xyz,float3(color.w)),0.0,1.0);
}
#else
{
#if (sc_BlendMode_AlphaTest)
{
shadowAlpha=1.0;
}
#else
{
#if (sc_BlendMode_Multiply)
{
shadowAlpha=(1.0-dot(color.xyz,float3(0.33333)))*color.w;
}
#else
{
#if (sc_BlendMode_MultiplyOriginal)
{
shadowAlpha=(1.0-fast::clamp(dot(color.xyz,float3(1.0)),0.0,1.0))*color.w;
}
#else
{
#if (sc_BlendMode_ColoredGlass)
{
shadowAlpha=fast::clamp(dot(color.xyz,float3(1.0)),0.0,1.0)*color.w;
}
#else
{
#if (sc_BlendMode_Add)
{
shadowAlpha=fast::clamp(dot(color.xyz,float3(1.0)),0.0,1.0);
}
#else
{
#if (sc_BlendMode_AddWithAlphaFactor)
{
shadowAlpha=fast::clamp(dot(color.xyz,float3(1.0)),0.0,1.0)*color.w;
}
#else
{
#if (sc_BlendMode_Screen)
{
shadowAlpha=dot(color.xyz,float3(0.33333))*color.w;
}
#else
{
#if (sc_BlendMode_Min)
{
shadowAlpha=1.0-fast::clamp(dot(color.xyz,float3(1.0)),0.0,1.0);
}
#else
{
#if (sc_BlendMode_Max)
{
shadowAlpha=fast::clamp(dot(color.xyz,float3(1.0)),0.0,1.0);
}
#endif
}
#endif
}
#endif
}
#endif
}
#endif
}
#endif
}
#endif
}
#endif
}
#endif
}
#endif
}
#endif
}
#endif
return shadowAlpha;
}

float4 evaluateShadowCasterColor(thread const float4& shadowCasterColor,constant LibraryUniforms& libraryUniformsObj)
{
float4 param=shadowCasterColor;
float alpha=getShadowAlpha(param);
float shadowTerm=libraryUniformsObj.sc_ShadowDensity*alpha;
float3 shadowColor=mix(libraryUniformsObj.sc_ShadowColor.xyz,libraryUniformsObj.sc_ShadowColor.xyz*shadowCasterColor.xyz,float3(libraryUniformsObj.sc_ShadowColor.w));
return float4(shadowColor.x,shadowColor.y,shadowColor.z,shadowTerm);
}

float getShadowTexClipFactor(thread float2& varShadowTex)
{
float2 uv=abs(varShadowTex-float2(0.5));
float clipFactor=fast::max(uv.x,uv.y);
return step(clipFactor,0.5);
}

float3 evaluateShadow(thread int& varStereoViewID,constant LibraryUniforms& libraryUniformsObj,thread texture2d_array<float> sc_ShadowTextureArrSC,thread const sampler sc_ShadowTextureArrSCSmplr,thread texture2d<float> sc_ShadowTexture,thread const sampler sc_ShadowTextureSmplr,thread float2& varShadowTex)
{
float2 param=varShadowTex;
float4 shadowSample=sc_ShadowTextureSampleView(param,varStereoViewID,libraryUniformsObj,sc_ShadowTextureArrSC,sc_ShadowTextureArrSCSmplr,sc_ShadowTexture,sc_ShadowTextureSmplr)*getShadowTexClipFactor(varShadowTex);
float3 shadowColor=mix(libraryUniformsObj.sc_ShadowColor.xyz,libraryUniformsObj.sc_ShadowColor.xyz*shadowSample.xyz,float3(libraryUniformsObj.sc_ShadowColor.w));
float shadowDensity=shadowSample.w*libraryUniformsObj.sc_ShadowDensity;
return mix(float3(1.0),shadowColor,float3(shadowDensity));
}

} // FRAGMENT SHADER
