#pragma once
#include <metal_stdlib>
#include <simd/simd.h>
#include "std2_shadows.metal"

using namespace metal;

namespace SNAP_VS {


//SG_REFLECTION_BEGIN
//attribute vec4 boneData:5:
//attribute vec3 blendShape0Pos:6:
//attribute vec3 blendShape0Normal:12:
//attribute vec3 blendShape1Pos:7:
//attribute vec3 blendShape1Normal:13:
//attribute vec3 blendShape2Pos:8:
//attribute vec3 blendShape2Normal:14:
//attribute vec3 blendShape3Pos:9:
//attribute vec3 blendShape4Pos:10:
//attribute vec3 blendShape5Pos:11:
//attribute vec4 position:0:
//attribute vec3 normal:1:
//attribute vec4 tangent:2:
//attribute vec2 texture0:3:
//attribute vec2 texture1:4:
//SG_REFLECTION_END

struct sc_Vertex_t
{
float4 position;
float3 normal;
float3 tangent;
float2 texture0;
float2 texture1;
};

#ifndef STD_DISABLE_VERTEX_NORMAL
#define STD_DISABLE_VERTEX_NORMAL 0
#elif STD_DISABLE_VERTEX_NORMAL==1
#undef STD_DISABLE_VERTEX_NORMAL
#define STD_DISABLE_VERTEX_NORMAL 1
#endif
#ifndef STD_DISABLE_VERTEX_TANGENT
#define STD_DISABLE_VERTEX_TANGENT 0
#elif STD_DISABLE_VERTEX_TANGENT==1
#undef STD_DISABLE_VERTEX_TANGENT
#define STD_DISABLE_VERTEX_TANGENT 1
#endif
#ifndef sc_VertexBlending
#define sc_VertexBlending 0
#elif sc_VertexBlending==1
#undef sc_VertexBlending
#define sc_VertexBlending 1
#endif
#ifndef sc_VertexBlendingUseNormals
#define sc_VertexBlendingUseNormals 0
#elif sc_VertexBlendingUseNormals==1
#undef sc_VertexBlendingUseNormals
#define sc_VertexBlendingUseNormals 1
#endif
#ifndef SC_DISABLE_FRUSTUM_CULLING
#define SC_DISABLE_FRUSTUM_CULLING 0
#elif SC_DISABLE_FRUSTUM_CULLING==1
#undef SC_DISABLE_FRUSTUM_CULLING
#define SC_DISABLE_FRUSTUM_CULLING 1
#endif
#ifndef STD_DISABLE_VERTEX_TEXTURE0
#define STD_DISABLE_VERTEX_TEXTURE0 0
#elif STD_DISABLE_VERTEX_TEXTURE0==1
#undef STD_DISABLE_VERTEX_TEXTURE0
#define STD_DISABLE_VERTEX_TEXTURE0 1
#endif
#ifndef STD_DISABLE_VERTEX_TEXTURE1
#define STD_DISABLE_VERTEX_TEXTURE1 0
#elif STD_DISABLE_VERTEX_TEXTURE1==1
#undef STD_DISABLE_VERTEX_TEXTURE1
#define STD_DISABLE_VERTEX_TEXTURE1 1
#endif
#ifndef sc_DepthBufferMode
#define sc_DepthBufferMode 0
#endif
#ifndef sc_TAAEnabled
#define sc_TAAEnabled 0
#elif sc_TAAEnabled==1
#undef sc_TAAEnabled
#define sc_TAAEnabled 1
#endif

int sc_GetLocalInstanceID(thread uint& gl_InstanceIndex)
{
return gl_InstanceIndex;
}

void sc_SetClipDistancePlatform(thread const float& dstClipDistance)
{
}

void sc_SetClipDistance(thread const float& dstClipDistance,thread float& varClipDistance)
{
#if (sc_StereoRendering_IsClipDistanceEnabled==1)
{
float param=dstClipDistance;
sc_SetClipDistancePlatform(param);
}
#else
{
varClipDistance=dstClipDistance;
}
#endif
}

void sc_SetClipDistance(thread const float4& clipPosition,constant LibraryUniforms& libraryUniformsObj,thread float& varClipDistance)
{
#if (sc_StereoRenderingMode==1)
{
float dstClipDistance=dot(clipPosition,libraryUniformsObj.sc_StereoClipPlanes[sc_StereoViewID]);
float param=dstClipDistance;
sc_SetClipDistance(param,varClipDistance);
}
#endif
}

void sc_SetClipPosition(thread const float4& clipPosition,constant LibraryUniforms& libraryUniformsObj,thread float& varClipDistance,thread int& varStereoViewID,thread float4& gl_Position)
{
#if (sc_StereoRenderingMode>0)
{
varStereoViewID=sc_StereoViewID;
}
#endif
float4 param=clipPosition;
sc_SetClipDistance(param,libraryUniformsObj,varClipDistance);
gl_Position=clipPosition;
}

float3 skinVertexPosition(thread const int& i,thread const float4& v,constant LibraryUniforms& libraryUniformsObj)
{
#if (sc_SkinBonesCount>0)
{
int j=3*i;
float4 m0=libraryUniformsObj.sc_BoneMatrices[j+0];
float4 m1=libraryUniformsObj.sc_BoneMatrices[j+1];
float4 m2=libraryUniformsObj.sc_BoneMatrices[j+2];
return float3(dot(m0,v),dot(m1,v),dot(m2,v));
}
#else
{
return v.xyz;
}
#endif
}

void sc_SkinVertex(thread sc_Vertex_t& v,constant LibraryUniforms& libraryUniformsObj,thread float4& boneData)
{
#if (sc_SkinBonesCount>0)
{
float4 weights=float4(1.0,fract(boneData.yzw));
weights.x-=dot(weights.yzw,float3(1.0));
int index0=int(boneData.x);
int index1=int(boneData.y);
int index2=int(boneData.z);
int index3=int(boneData.w);
int param=index0;
float4 param_1=v.position;
int param_2=index1;
float4 param_3=v.position;
int param_4=index2;
float4 param_5=v.position;
int param_6=index3;
float4 param_7=v.position;
float3 _1549=(((skinVertexPosition(param,param_1,libraryUniformsObj)*weights.x)+(skinVertexPosition(param_2,param_3,libraryUniformsObj)*weights.y))+(skinVertexPosition(param_4,param_5,libraryUniformsObj)*weights.z))+(skinVertexPosition(param_6,param_7,libraryUniformsObj)*weights.w);
v.position=float4(_1549.x,_1549.y,_1549.z,v.position.w);
float3x3 normalMatrix0;
float3x3 normalMatrix1;
float3x3 normalMatrix2;
float3x3 normalMatrix3;
#if ((!STD_DISABLE_VERTEX_NORMAL)||(!STD_DISABLE_VERTEX_TANGENT))
{
normalMatrix0=libraryUniformsObj.sc_SkinBonesNormalMatrices[index0];
normalMatrix1=libraryUniformsObj.sc_SkinBonesNormalMatrices[index1];
normalMatrix2=libraryUniformsObj.sc_SkinBonesNormalMatrices[index2];
normalMatrix3=libraryUniformsObj.sc_SkinBonesNormalMatrices[index3];
}
#endif
#if (!STD_DISABLE_VERTEX_NORMAL)
{
v.normal=((((normalMatrix0*v.normal)*weights.x)+((normalMatrix1*v.normal)*weights.y))+((normalMatrix2*v.normal)*weights.z))+((normalMatrix3*v.normal)*weights.w);
}
#endif
#if (!STD_DISABLE_VERTEX_TANGENT)
{
v.tangent=((((normalMatrix0*v.tangent)*weights.x)+((normalMatrix1*v.tangent)*weights.y))+((normalMatrix2*v.tangent)*weights.z))+((normalMatrix3*v.tangent)*weights.w);
}
#endif
}
#endif
}

void blendTargetShape(thread sc_Vertex_t& v,thread const float3& position,thread const float& weight)
{
float3 _1654=v.position.xyz+(position*weight);
v.position=float4(_1654.x,_1654.y,_1654.z,v.position.w);
}

void blendTargetShapeWithNormal(thread sc_Vertex_t& v,thread const float3& position,thread const float3& normal,thread const float& weight)
{
sc_Vertex_t param=v;
float3 param_1=position;
float param_2=weight;
blendTargetShape(param,param_1,param_2);
v=param;
v.normal+=(normal*weight);
}

void sc_BlendVertex(thread sc_Vertex_t& v,constant LibraryUniforms& libraryUniformsObj,thread float3& blendShape0Pos,thread float3& blendShape0Normal,thread float3& blendShape1Pos,thread float3& blendShape1Normal,thread float3& blendShape2Pos,thread float3& blendShape2Normal,thread float3& blendShape3Pos,thread float3& blendShape4Pos,thread float3& blendShape5Pos)
{
#if (sc_VertexBlending)
{
#if (sc_VertexBlendingUseNormals)
{
sc_Vertex_t param=v;
float3 param_1=blendShape0Pos;
float3 param_2=blendShape0Normal;
float param_3=libraryUniformsObj.weights0.x;
blendTargetShapeWithNormal(param,param_1,param_2,param_3);
v=param;
sc_Vertex_t param_4=v;
float3 param_5=blendShape1Pos;
float3 param_6=blendShape1Normal;
float param_7=libraryUniformsObj.weights0.y;
blendTargetShapeWithNormal(param_4,param_5,param_6,param_7);
v=param_4;
sc_Vertex_t param_8=v;
float3 param_9=blendShape2Pos;
float3 param_10=blendShape2Normal;
float param_11=libraryUniformsObj.weights0.z;
blendTargetShapeWithNormal(param_8,param_9,param_10,param_11);
v=param_8;
}
#else
{
sc_Vertex_t param_12=v;
float3 param_13=blendShape0Pos;
float param_14=libraryUniformsObj.weights0.x;
blendTargetShape(param_12,param_13,param_14);
v=param_12;
sc_Vertex_t param_15=v;
float3 param_16=blendShape1Pos;
float param_17=libraryUniformsObj.weights0.y;
blendTargetShape(param_15,param_16,param_17);
v=param_15;
sc_Vertex_t param_18=v;
float3 param_19=blendShape2Pos;
float param_20=libraryUniformsObj.weights0.z;
blendTargetShape(param_18,param_19,param_20);
v=param_18;
sc_Vertex_t param_21=v;
float3 param_22=blendShape3Pos;
float param_23=libraryUniformsObj.weights0.w;
blendTargetShape(param_21,param_22,param_23);
v=param_21;
sc_Vertex_t param_24=v;
float3 param_25=blendShape4Pos;
float param_26=libraryUniformsObj.weights1.x;
blendTargetShape(param_24,param_25,param_26);
v=param_24;
sc_Vertex_t param_27=v;
float3 param_28=blendShape5Pos;
float param_29=libraryUniformsObj.weights1.y;
blendTargetShape(param_27,param_28,param_29);
v=param_27;
}
#endif
}
#endif
}

float4 sc_ApplyScreenSpaceInstancedClippedShift(thread float4& screenPosition)
{
#if (sc_StereoRenderingMode==1)
{
float viewIndex=float(sc_GetStereoViewIndex());
screenPosition.y=(screenPosition.y*0.5)+(0.5-viewIndex);
}
#endif
return screenPosition;
}

float4 sc_ObjectToScreen(thread float4& objectPosition,constant LibraryUniforms& libraryUniformsObj)
{
#if (sc_RenderingSpace==3)
{
float4 param=objectPosition;
float4 _1801=sc_ApplyScreenSpaceInstancedClippedShift(param);
return _1801;
}
#else
{
#if (sc_RenderingSpace==2)
{
return libraryUniformsObj.sc_ViewProjectionMatrixArray[sc_GetStereoViewIndex()]*objectPosition;
}
#else
{
#if (sc_RenderingSpace==1)
{
return libraryUniformsObj.sc_ModelViewProjectionMatrixArray[sc_GetStereoViewIndex()]*objectPosition;
}
#else
{
#if (sc_RenderingSpace==4)
{
objectPosition=(libraryUniformsObj.sc_ModelViewMatrixArray[sc_GetStereoViewIndex()]*objectPosition)*float4(1.0/libraryUniformsObj.sc_Camera.aspect,1.0,1.0,1.0);
float4 param_1=objectPosition;
float4 _1844=sc_ApplyScreenSpaceInstancedClippedShift(param_1);
return _1844;
}
#else
{
return objectPosition;
}
#endif
}
#endif
}
#endif
}
#endif
}

float4 sc_ObjectToView(thread const float4& objectPosition,constant LibraryUniforms& libraryUniformsObj)
{
#if (sc_RenderingSpace==3)
{
return libraryUniformsObj.sc_ProjectionMatrixInverseArray[sc_GetStereoViewIndex()]*objectPosition;
}
#else
{
#if (sc_RenderingSpace==2)
{
return libraryUniformsObj.sc_ViewMatrixArray[sc_GetStereoViewIndex()]*objectPosition;
}
#else
{
#if (sc_RenderingSpace==1)
{
return libraryUniformsObj.sc_ModelViewMatrixArray[sc_GetStereoViewIndex()]*objectPosition;
}
#else
{
return objectPosition;
}
#endif
}
#endif
}
#endif
}

sc_Vertex_t sc_LoadVertexAttributes(constant LibraryUniforms& libraryUniformsObj,thread float4& position,thread float3& normal,thread float4& tangent,thread float2& texture0,thread float2& texture1)
{
sc_Vertex_t v;
v.position=position;
#if (sc_IsEditor&&SC_DISABLE_FRUSTUM_CULLING)
{
v.position.x+=libraryUniformsObj.sc_DisableFrustumCullingMarker;
}
#endif
#if (!STD_DISABLE_VERTEX_NORMAL)
{
v.normal=normal;
}
#endif
#if (!STD_DISABLE_VERTEX_TANGENT)
{
v.tangent=tangent.xyz;
}
#endif
#if (!STD_DISABLE_VERTEX_TEXTURE0)
{
v.texture0=texture0;
}
#endif
#if (!STD_DISABLE_VERTEX_TEXTURE1)
{
v.texture1=texture1;
}
#endif
return v;
}

float4 applyDepthAlgorithm(thread float4& screenPosition,constant LibraryUniforms& libraryUniformsObj)
{
#if (sc_DepthBufferMode==1)
{
float fCoefficient=2.0/log2(libraryUniformsObj.sc_Camera.clipPlanes.y+1.0);
screenPosition.z=((log2(fast::max(libraryUniformsObj.sc_Camera.clipPlanes.x,1.0+screenPosition.w))*fCoefficient)-1.0)*screenPosition.w;
}
#endif
return screenPosition;
}

void sc_ProcessVertex(thread sc_Vertex_t& v,constant LibraryUniforms& libraryUniformsObj,thread float& varClipDistance,thread int& varStereoViewID,thread float4& gl_Position,thread float4& boneData,thread float3& blendShape0Pos,thread float3& blendShape0Normal,thread float3& blendShape1Pos,thread float3& blendShape1Normal,thread float3& blendShape2Pos,thread float3& blendShape2Normal,thread float3& blendShape3Pos,thread float3& blendShape4Pos,thread float3& blendShape5Pos,thread float4& tangent,thread float3& varPos,thread float3& varNormal,thread float4& varTangent,thread float4& varPackedTex,thread float4& varScreenPos,thread float2& varScreenTexturePos,thread float2& varShadowTex,thread float& varViewSpaceDepth)
{
sc_Vertex_t param=v;
sc_BlendVertex(param,libraryUniformsObj,blendShape0Pos,blendShape0Normal,blendShape1Pos,blendShape1Normal,blendShape2Pos,blendShape2Normal,blendShape3Pos,blendShape4Pos,blendShape5Pos);
v=param;
sc_Vertex_t param_1=v;
sc_SkinVertex(param_1,libraryUniformsObj,boneData);
v=param_1;
float4 param_2=v.position;
float4 _1971=sc_ObjectToScreen(param_2,libraryUniformsObj);
float4 screenPosition=_1971;
#if ((sc_RenderingSpace==3)||(sc_RenderingSpace==4))
{
varPos=screenPosition.xyz;
#if (!STD_DISABLE_VERTEX_NORMAL)
{
varNormal=normalize(v.normal);
}
#endif
#if (!STD_DISABLE_VERTEX_TANGENT)
{
float3 _1994=normalize(v.tangent);
varTangent=float4(_1994.x,_1994.y,_1994.z,varTangent.w);
}
#endif
}
#else
{
#if (sc_RenderingSpace==2)
{
varPos=v.position.xyz;
#if (!STD_DISABLE_VERTEX_NORMAL)
{
varNormal=normalize(v.normal);
}
#endif
#if (!STD_DISABLE_VERTEX_TANGENT)
{
float3 _2015=normalize(v.tangent);
varTangent=float4(_2015.x,_2015.y,_2015.z,varTangent.w);
}
#endif
}
#else
{
#if (sc_RenderingSpace==1)
{
varPos=(libraryUniformsObj.sc_ModelMatrix*v.position).xyz;
#if (!STD_DISABLE_VERTEX_NORMAL)
{
varNormal=normalize(libraryUniformsObj.sc_NormalMatrix*v.normal);
}
#endif
#if (!STD_DISABLE_VERTEX_TANGENT)
{
float3 _2046=normalize(libraryUniformsObj.sc_NormalMatrix*v.tangent);
varTangent=float4(_2046.x,_2046.y,_2046.z,varTangent.w);
}
#endif
}
#endif
}
#endif
}
#endif
#if (!STD_DISABLE_VERTEX_TANGENT)
{
varTangent.w=tangent.w;
}
#endif
#if (!STD_DISABLE_VERTEX_TEXTURE0)
{
varPackedTex=float4(v.texture0.x,v.texture0.y,varPackedTex.z,varPackedTex.w);
}
#endif
#if (!STD_DISABLE_VERTEX_TEXTURE1)
{
varPackedTex=float4(varPackedTex.x,varPackedTex.y,v.texture1.x,v.texture1.y);
}
#endif
varScreenPos=screenPosition;
float2 globalScreenCoords=((screenPosition.xy/float2(screenPosition.w))*0.5)+float2(0.5);
float2 param_3=globalScreenCoords;
varScreenTexturePos=sc_ScreenCoordsGlobalToView(param_3);
#if (sc_ProjectiveShadowsReceiver)
{
float4 param_4=v.position;
varShadowTex=getProjectedTexCoords(param_4,libraryUniformsObj);
}
#endif
#if ((sc_OITDepthGatherPass||sc_OITCompositingPass)||sc_OITDepthBoundsPass)
{
float4 param_5=v.position;
varViewSpaceDepth=-sc_ObjectToView(param_5,libraryUniformsObj).z;
}
#endif
float4 param_6=screenPosition;
float4 _2111=applyDepthAlgorithm(param_6,libraryUniformsObj);
screenPosition=_2111;
#if (sc_TAAEnabled)
{
float2 _2124=screenPosition.xy+(libraryUniformsObj.sc_TAAJitterOffset*screenPosition.w);
screenPosition=float4(_2124.x,_2124.y,screenPosition.z,screenPosition.w);
}
#endif
float4 clipPosition=screenPosition*1.0;
float4 param_7=clipPosition;
sc_SetClipPosition(param_7,libraryUniformsObj,varClipDistance,varStereoViewID,gl_Position);
}

} // VERTEX SHADER


namespace SNAP_FS {


//SG_REFLECTION_BEGIN
//SG_REFLECTION_END

#ifndef sc_DepthBufferMode
#define sc_DepthBufferMode 0
#endif
#ifndef sc_TAAEnabled
#define sc_TAAEnabled 0
#elif sc_TAAEnabled==1
#undef sc_TAAEnabled
#define sc_TAAEnabled 1
#endif
#ifndef sc_VertexBlending
#define sc_VertexBlending 0
#elif sc_VertexBlending==1
#undef sc_VertexBlending
#define sc_VertexBlending 1
#endif
#ifndef sc_VertexBlendingUseNormals
#define sc_VertexBlendingUseNormals 0
#elif sc_VertexBlendingUseNormals==1
#undef sc_VertexBlendingUseNormals
#define sc_VertexBlendingUseNormals 1
#endif
#ifndef STD_DISABLE_VERTEX_NORMAL
#define STD_DISABLE_VERTEX_NORMAL 0
#elif STD_DISABLE_VERTEX_NORMAL==1
#undef STD_DISABLE_VERTEX_NORMAL
#define STD_DISABLE_VERTEX_NORMAL 1
#endif
#ifndef STD_DISABLE_VERTEX_TANGENT
#define STD_DISABLE_VERTEX_TANGENT 0
#elif STD_DISABLE_VERTEX_TANGENT==1
#undef STD_DISABLE_VERTEX_TANGENT
#define STD_DISABLE_VERTEX_TANGENT 1
#endif
#ifndef STD_DISABLE_VERTEX_TEXTURE0
#define STD_DISABLE_VERTEX_TEXTURE0 0
#elif STD_DISABLE_VERTEX_TEXTURE0==1
#undef STD_DISABLE_VERTEX_TEXTURE0
#define STD_DISABLE_VERTEX_TEXTURE0 1
#endif
#ifndef STD_DISABLE_VERTEX_TEXTURE1
#define STD_DISABLE_VERTEX_TEXTURE1 0
#elif STD_DISABLE_VERTEX_TEXTURE1==1
#undef STD_DISABLE_VERTEX_TEXTURE1
#define STD_DISABLE_VERTEX_TEXTURE1 1
#endif
#ifndef SC_DISABLE_FRUSTUM_CULLING
#define SC_DISABLE_FRUSTUM_CULLING 0
#elif SC_DISABLE_FRUSTUM_CULLING==1
#undef SC_DISABLE_FRUSTUM_CULLING
#define SC_DISABLE_FRUSTUM_CULLING 1
#endif

} // FRAGMENT SHADER
