//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------
#define BASECOLOR mainColor

#define BASETEXNAME mainTexture
#define BASETEXNAMESAMPLER mainTextureSampler
#define BASETEXNAMETRANSFORM mainTextureTransform

#define MASKTEXNAME maskTexture
#define MASKTEXNAMESAMPLER maskTextureSampler
#define MASKTEXNAMETRANSFORM maskTextureTransform

#if defined(CMDLINE)
#include "metal_std.metal"
#include "metalstd_vs.metal"
#include "metalstd_fs.metal"
#include "metalstd_shadows.metal"
#endif
#else
#include <metal_std.metal>
#include <metalstd_vs.metal>
#include <metalstd_fs.metal>
#include <metalstd_shadows.metal>
#endif

constant bool NOTEXTURE [[function_constant(FUNCONST_VS_USER_BIAS+0)]];
constant bool HASMAINTEXTURE  = (NOTEXTURE==false);
constant bool NOMASK [[function_constant(FUNCONST_VS_USER_BIAS+1)]];
constant bool HASMASK = (NOMASK==false);

struct FSUniforms
{
  float4 BASECOLOR;
};

VSMAIN()
{
  sc_Vertex_t v = sc_LoadVertexAttributes(vtx);

  if(NOTEXTURE)
  {
    v.texture0 = float2(0.0);
  }
  else
  {
    v.texture0 = float2((uniforms.mainTextureTransform * float3(v.texture0, 1.0)).xy);
  }

  if(NOMASK)
  {
    v.texture1 = float2(0.0);
  }
  else
  {
    v.texture1 = float2((uniforms.maskTextureTransform * float3(v.texture1, 1.0)).xy);
  }

  sc_ProcessVertex(v);
}

// fragment shader function
fragment half4 fs_main(Varyings in [[stage_in]],
                       constant FSUniforms& uniforms [[ buffer(2) ]],
                       constant FSShadowsUniforms& shadowsUniforms[[function_constant(needShadowsUnfiforms),buffer(3)]],
                       DECLARETEXTUREOPTIONAL(BASETEXNAME,BASETEXNAMESAMPLER,HASMAINTEXTURE),
                       DECLARETEXTUREOPTIONAL(MASKTEXNAME,MASKTEXNAMESAMPLER,HASMASK),
                       DECLARETEXTUREOPTIONAL(sc_ShadowTexture,sc_ShadowTextureSampler,sc_ProjectiveShadowsReceiver))
{
  half4 color;
  if( NOTEXTURE )
  {
    color = half4(uniforms.mainColor);
  }
  else
  {
    color = sampleTextureSimple(mainTexture,mainTextureSampler, float2(in.varTex0)) * half4(uniforms.mainColor);
  }
  
  half maskAlpha;
  if(NOMASK)
  {
    maskAlpha = 1.0;
  }
  else
  {
    maskAlpha = sampleTextureSimple(maskTexture,maskTextureSampler, in.varTex1).r;
  }
  color.a *= maskAlpha;
    
  if(sc_ProjectiveShadowsReceiver)
  {
    color.rgb = evaluateShadow(sc_ShadowTexture,sc_ShadowTextureSampler,uniforms,color.rgb,in.varShadowTex);
  }
    
  if( sc_ProjectiveShadowsCaster )
  {
    half shadowAlpha = getShadowAlpha(color, uniforms.alphaTestThreshold);
    color = evaluateShadowCasterColor(color.rgb, shadowAlpha,shadowsUniforms);
  }

  return color;
};

